@@ -1,11 +1,7 @@
-// "False positive, see https://github.com/rust-lang/rust-clippy/issues/13358#issuecomment-2767079754"
-#![allow(unfulfilled_lint_expectations)]
-#![expect(clippy::wildcard_imports)]
-
 use std::fmt::{Display, Write as _};
 use std::io::{self, BufRead, BufReader, ErrorKind, Read, Seek, Write};
 use std::iter::{self, repeat, successors};
-use std::{fs::File, path::Path, process::Command, thread, time::Instant};
+use std::{fs::File, path::Path, process::Command, time::Instant};
 
 use crate::row::{HlState, Row};
 use crate::{Config, Error, ansi_escape::*, syntax::Conf as SyntaxConf, sys, terminal};
@@ -25,6 +21,7 @@ const EXECUTE: u8 = ctrl_key(b'E');
 const REMOVE_LINE: u8 = ctrl_key(b'R');
 const BACKSPACE: u8 = 127;
 
+const WELCOME_MESSAGE: &str = concat!("Kibi ", env!("KIBI_VERSION"));
 const HELP_MESSAGE: &str = "^S save | ^Q quit | ^F find | ^G go to | ^D duplicate | ^E execute | \
                             ^C copy | ^X cut | ^V paste";
 
@@ -107,8 +104,8 @@ pub struct Editor {
     dirty: bool,
     /// The configuration for the editor.
     config: Config,
-    /// The number of warnings remaining before we can quit without saving.
-    /// Defaults to `config.quit_times`, then decreases to 0.
+    /// The number of consecutive times the user has tried to quit without
+    /// saving. After `config.quit_times`, the program will exit.
     quit_times: usize,
     /// The file name. If None, the user will be prompted for a file name the
     /// first time they try to save.
@@ -120,11 +117,10 @@ pub struct Editor {
     syntax: SyntaxConf,
     /// The number of bytes contained in `rows`. This excludes new lines.
     n_bytes: u64,
-    /// The original terminal mode. It will be restored when the `Editor`
-    /// instance is dropped.
-    orig_term_mode: Option<sys::TermMode>,
     /// The copied buffer of a row
     copied_row: Vec<u8>,
+    /// Whether to use ANSI color escape codes for rendering
+    use_color: bool,
 }
 
 /// Describes a status message, shown at the bottom at the screen.
@@ -154,7 +150,7 @@ fn format_size(n: u64) -> String {
     format!("{}.{:02}{}B", q / 100, q % 100, b" kMGTPEZ"[i as usize] as char)
 }
 
-/// Return an Arrow Key given and ANSI code.
+/// Return an Arrow Key given an ANSI code.
 ///
 /// The argument must be a valide arrow key ANSI code (`a`, `b`, `c` or `d`),
 /// case-insensitive).
@@ -169,27 +165,6 @@ fn get_akey(c: u8) -> AKey {
 }
 
 impl Editor {
-    /// Initialize the text editor.
-    ///
-    /// # Errors
-    ///
-    /// Will return `Err` if an error occurs when enabling termios raw mode,
-    /// creating the signal hook or when obtaining the terminal window size.
-    pub fn new(config: Config) -> Result<Self, Error> {
-        sys::register_winsize_change_signal_handler()?;
-        let mut editor = Self::default();
-        (editor.quit_times, editor.config) = (config.quit_times, config);
-
-        // Enable raw mode and store the original (non-raw) terminal mode.
-        editor.orig_term_mode = Some(sys::enable_raw_mode()?);
-        print!("{USE_ALTERNATE_SCREEN}");
-
-        editor.update_window_size()?;
-        set_status!(editor, "{HELP_MESSAGE}");
-
-        Ok(editor)
-    }
-
     /// Return the current row if the cursor points to an existing row, `None`
     /// otherwise.
     fn current_row(&self) -> Option<&Row> { self.rows.get(self.cursor.y) }
@@ -201,26 +176,27 @@ impl Editor {
 
     /// Move the cursor following an arrow key (← → ↑ ↓).
     fn move_cursor(&mut self, key: &AKey, ctrl: bool) {
-        let mut cursor_x = self.cursor.x;
         match (key, self.current_row()) {
             (AKey::Left, Some(row)) if self.cursor.x > 0 => {
-                cursor_x -= row.get_char_size(row.cx2rx[cursor_x] - 1);
+                let mut cursor_x = self.cursor.x - row.get_char_size(row.cx2rx[self.cursor.x] - 1);
                 // ← moving to previous word
                 while ctrl && cursor_x > 0 && row.chars[cursor_x - 1] != b' ' {
                     cursor_x -= row.get_char_size(row.cx2rx[cursor_x] - 1);
                 }
+                self.cursor.x = cursor_x;
             }
             // ← at the beginning of the line: move to the end of the previous line. The x
             // position will be adjusted after this `match` to accommodate the current row
             // length, so we can just set here to the maximum possible value here.
             (AKey::Left, _) if self.cursor.y > 0 =>
-                (self.cursor.y, cursor_x) = (self.cursor.y - 1, usize::MAX),
+                (self.cursor.y, self.cursor.x) = (self.cursor.y - 1, usize::MAX),
             (AKey::Right, Some(row)) if self.cursor.x < row.chars.len() => {
-                cursor_x += row.get_char_size(row.cx2rx[cursor_x]);
+                let mut cursor_x = self.cursor.x + row.get_char_size(row.cx2rx[self.cursor.x]);
                 // → moving to next word
                 while ctrl && cursor_x < row.chars.len() && row.chars[cursor_x] != b' ' {
                     cursor_x += row.get_char_size(row.cx2rx[cursor_x]);
                 }
+                self.cursor.x = cursor_x;
             }
             (AKey::Right, Some(_)) => self.cursor.move_to_next_line(),
             // TODO: For Up and Down, move self.cursor.x to be consistent with tabs and UTF-8
@@ -229,7 +205,6 @@ impl Editor {
             (AKey::Down, Some(_)) => self.cursor.y += 1,
             _ => (),
         }
-        self.cursor.x = cursor_x;
         self.update_cursor_x_position();
     }
 
@@ -253,9 +228,9 @@ impl Editor {
                 self.update_window_size()?;
                 self.refresh_screen()?;
             }
+            // Match on the next byte received or, if the first byte is <ESC> ('\x1b'), on
+            // the next few bytes.
             if let Some(a) = bytes.next().transpose()? {
-                // Match on the next byte received or, if the first byte is <ESC> ('\x1b'), on
-                // the next few bytes.
                 if a != b'\x1b' {
                     return Ok(Key::Char(a));
                 }
@@ -398,7 +373,7 @@ impl Editor {
             self.update_row(self.cursor.y, false);
             // The number of rows has changed. The left padding may need to be updated.
             self.update_screen_cols();
-            (self.dirty, self.cursor.y) = (self.dirty, self.cursor.y - 1);
+            (self.dirty, self.cursor.y) = (true, self.cursor.y - 1);
         } else if self.cursor.y == self.rows.len() {
             // If the cursor is located after the last row, pressing backspace is equivalent
             // to pressing the left arrow key.
@@ -409,9 +384,10 @@ impl Editor {
     fn delete_current_row(&mut self) {
         if self.cursor.y < self.rows.len() {
             self.rows[self.cursor.y].chars.clear();
-            self.update_row(self.cursor.y, false);
-            self.cursor.move_to_next_line();
+            self.cursor.x = 0;
+            self.cursor.y = std::cmp::min(self.cursor.y + 1, self.rows.len() - 1);
             self.delete_char();
+            self.cursor.x = 0;
         }
     }
 
@@ -431,14 +407,10 @@ impl Editor {
             return;
         }
         self.n_bytes += self.copied_row.len() as u64;
-        if self.cursor.y == self.rows.len() {
-            self.rows.push(Row::new(self.copied_row.clone()));
-        } else {
-            self.rows.insert(self.cursor.y + 1, Row::new(self.copied_row.clone()));
-        }
-        self.update_row(self.cursor.y + usize::from(self.cursor.y + 1 != self.rows.len()), false);
-        (self.cursor.y, self.dirty) = (self.cursor.y + 1, true);
-        // The line number has changed
+        let y = (self.cursor.y + 1).min(self.rows.len());
+        self.rows.insert(y, Row::new(self.copied_row.clone()));
+        self.update_row(y, false);
+        (self.cursor.y, self.dirty) = (y, true);
         self.update_screen_cols();
     }
 
@@ -518,18 +490,19 @@ impl Editor {
     }
 
     /// Draw the left part of the screen: line numbers and vertical bar.
-    fn draw_left_padding<T: Display>(&self, buffer: &mut String, val: T) -> Result<(), Error> {
+    fn draw_left_padding<T: Display>(&self, buffer: &mut String, val: T) {
         if self.ln_pad >= 2 {
-            // \x1b[38;5;240m: Dark grey color; \u{2502}: pipe "│"
-            write!(buffer, "\x1b[38;5;240m{:>2$} \u{2502}{}", val, RESET_FMT, self.ln_pad - 2)?;
+            // \u{2502}: pipe "│"
+            let s = format!("{:>1$} \u{2502}", val, self.ln_pad - 2);
+            // \x1b[38;5;240m: Dark grey color
+            push_colored(buffer, "\x1b[38;5;240m", &s, self.use_color);
         }
-        Ok(())
     }
 
     /// Return whether the file being edited is empty or not. If there is more
     /// than one row, even if all the rows are empty, `is_empty` returns
     /// `false`, since the text contains new lines.
-    fn is_empty(&self) -> bool { self.rows.len() <= 1 && self.n_bytes == 0 }
+    const fn is_empty(&self) -> bool { self.rows.len() <= 1 && self.n_bytes == 0 }
 
     /// Draw rows of text and empty rows on the terminal, by adding characters
     /// to the buffer.
@@ -539,14 +512,13 @@ impl Editor {
             buffer.push_str(CLEAR_LINE_RIGHT_OF_CURSOR);
             if let Some(row) = row {
                 // Draw a row of text
-                self.draw_left_padding(buffer, i + 1)?;
-                row.draw(self.cursor.coff, self.screen_cols, buffer)?;
+                self.draw_left_padding(buffer, i + 1);
+                row.draw(self.cursor.coff, self.screen_cols, buffer, self.use_color);
             } else {
                 // Draw an empty row
-                self.draw_left_padding(buffer, '~')?;
+                self.draw_left_padding(buffer, '~');
                 if self.is_empty() && i == self.screen_rows / 3 {
-                    let welcome_message = concat!("Kibi ", env!("KIBI_VERSION"));
-                    write!(buffer, "{:^1$.1$}", welcome_message, self.screen_cols)?;
+                    write!(buffer, "{:^1$.1$}", WELCOME_MESSAGE, self.screen_cols)?;
                 }
             }
             buffer.push_str("\r\n");
@@ -555,7 +527,7 @@ impl Editor {
     }
 
     /// Draw the status bar on the terminal, by adding characters to the buffer.
-    fn draw_status_bar(&self, buffer: &mut String) -> Result<(), Error> {
+    fn draw_status_bar(&self, buffer: &mut String) {
         // Left part of the status bar
         let modified = if self.dirty { " (modified)" } else { "" };
         let mut left =
@@ -569,8 +541,7 @@ impl Editor {
 
         // Draw
         let rw = self.window_width.saturating_sub(left.len());
-        write!(buffer, "{REVERSE_VIDEO}{left}{right:>rw$.rw$}{RESET_FMT}\r\n")?;
-        Ok(())
+        push_colored(buffer, WBG, &format!("{left}{right:>rw$.rw$}\r\n"), self.use_color);
     }
 
     /// Draw the message bar on the terminal, by adding characters to the
@@ -589,7 +560,7 @@ impl Editor {
         self.cursor.scroll(self.rx(), self.screen_rows, self.screen_cols);
         let mut buffer = format!("{HIDE_CURSOR}{MOVE_CURSOR_TO_START}");
         self.draw_rows(&mut buffer)?;
-        self.draw_status_bar(&mut buffer)?;
+        self.draw_status_bar(&mut buffer);
         self.draw_message_bar(&mut buffer);
         let (cursor_x, cursor_y) = if self.prompt_mode.is_none() {
             // If not in prompt mode, position the cursor according to the `cursor`
@@ -610,7 +581,7 @@ impl Editor {
     /// switch to.
     fn process_keypress(&mut self, key: &Key) -> (bool, Option<PromptMode>) {
         // This won't be mutated, unless key is Key::Character(EXIT)
-        let mut quit_times = self.config.quit_times;
+        let mut reset_quit_times = true;
         let mut prompt_mode = None;
 
         match key {
@@ -635,12 +606,12 @@ impl Editor {
             }
             Key::Escape | Key::Char(REFRESH_SCREEN) => (),
             Key::Char(EXIT) => {
-                quit_times = self.quit_times - 1;
-                if !self.dirty || quit_times == 0 {
+                if !self.dirty || self.quit_times + 1 >= self.config.quit_times {
                     return (true, None);
                 }
-                let times = if quit_times > 1 { "times" } else { "time" };
-                set_status!(self, "Press Ctrl+Q {quit_times} more {times} to quit.");
+                let r = self.config.quit_times - self.quit_times - 1;
+                set_status!(self, "Press Ctrl+Q {0} more time{1:.2$} to quit.", r, "s", r - 1);
+                reset_quit_times = false;
             }
             Key::Char(SAVE) => match self.file_name.take() {
                 // TODO: Can we avoid using take() then reassigning the value to file_name?
@@ -663,7 +634,7 @@ impl Editor {
             Key::Char(EXECUTE) => prompt_mode = Some(PromptMode::Execute(String::new())),
             Key::Char(c) => self.insert_byte(*c),
         }
-        self.quit_times = quit_times;
+        self.quit_times = if reset_quit_times { 0 } else { self.quit_times + 1 };
         (false, prompt_mode)
     }
 
@@ -697,7 +668,10 @@ impl Editor {
     /// # Errors
     ///
     /// Will Return `Err` if any error occur.
-    pub fn run(&mut self, file_name: Option<&str>) -> Result<(), Error> {
+    pub fn run<I: BufRead>(&mut self, file_name: Option<&str>, input: &mut I) -> Result<(), Error> {
+        self.update_window_size()?;
+        set_status!(self, "{HELP_MESSAGE}");
+
         if let Some(path) = file_name.map(sys::path) {
             self.syntax = SyntaxConf::find(&path.to_string_lossy(), &sys::data_dirs());
             self.load(path.as_path())?;
@@ -711,7 +685,7 @@ impl Editor {
                 set_status!(self, "{}", mode.status_msg());
             }
             self.refresh_screen()?;
-            let key = self.loop_until_keypress(&mut sys::stdin()?)?;
+            let key = self.loop_until_keypress(input)?;
             // TODO: Can we avoid using take()?
             self.prompt_mode = match self.prompt_mode.take() {
                 // process_keypress returns (should_quit, prompt_mode)
@@ -725,18 +699,35 @@ impl Editor {
     }
 }
 
-impl Drop for Editor {
-    #[expect(clippy::expect_used)]
-    /// When the editor is dropped, restore the original terminal mode.
-    fn drop(&mut self) {
-        if let Some(orig_term_mode) = self.orig_term_mode.take() {
-            sys::set_term_mode(&orig_term_mode).expect("Could not restore original terminal mode.");
-        }
-        if !thread::panicking() {
-            print!("{USE_MAIN_SCREEN}");
-            io::stdout().flush().expect("Could not flush stdout");
-        }
-    }
+/// Set up the terminal and run the text editor. If `file_name` is not None,
+/// load the file.
+///
+/// Update the panic hook to restore the terminal on panic.
+///
+/// # Errors
+///
+/// Will Return `Err` if any error occur when registering the window size signal
+/// handler, enabling raw mode, or running the editor.
+pub fn run<I: BufRead>(file_name: Option<&str>, input: &mut I) -> Result<(), Error> {
+    sys::register_winsize_change_signal_handler()?;
+    let orig_term_mode = sys::enable_raw_mode()?;
+    let mut editor = Editor { config: Config::load(), ..Default::default() };
+    editor.use_color = !std::env::var("NO_COLOR").is_ok_and(|val| !val.is_empty());
+
+    print!("{USE_ALTERNATE_SCREEN}");
+
+    let prev_hook = std::panic::take_hook();
+    std::panic::set_hook(Box::new(move |info| {
+        terminal::restore_terminal(&orig_term_mode).unwrap_or_else(|e| eprintln!("{e}"));
+        prev_hook(info);
+    }));
+
+    let result = editor.run(file_name, input);
+
+    // Restore the original terminal mode.
+    terminal::restore_terminal(&orig_term_mode)?;
+
+    result
 }
 
 /// The prompt mode.
@@ -869,6 +860,40 @@ mod tests {
     use rstest::rstest;
 
     use super::*;
+    use crate::syntax::HlType;
+
+    fn assert_row_chars_equal(editor: &Editor, expected: &[&[u8]]) {
+        assert_eq!(
+            editor.rows.len(),
+            expected.len(),
+            "editor has {} rows, expected {}",
+            editor.rows.len(),
+            expected.len()
+        );
+        for (i, (row, expected)) in editor.rows.iter().zip(expected).enumerate() {
+            assert_eq!(
+                row.chars,
+                *expected,
+                "comparing characters for row {}\n  left: {}\n  right: {}",
+                i,
+                String::from_utf8_lossy(&row.chars),
+                String::from_utf8_lossy(expected)
+            );
+        }
+    }
+
+    fn assert_row_synthax_highlighting_types_equal(editor: &Editor, expected: &[&[HlType]]) {
+        assert_eq!(
+            editor.rows.len(),
+            expected.len(),
+            "editor has {} rows, expected {}",
+            editor.rows.len(),
+            expected.len()
+        );
+        for (i, (row, expected)) in editor.rows.iter().zip(expected).enumerate() {
+            assert_eq!(row.hl, *expected, "comparing HlTypes for row {i}",);
+        }
+    }
 
     #[rstest]
     #[case(0, "0B")]
@@ -925,6 +950,7 @@ mod tests {
             assert_eq!(row.chars, []);
         }
     }
+
     #[test]
     fn editor_delete_char() {
         let mut editor = Editor::default();
@@ -932,23 +958,37 @@ mod tests {
             editor.insert_byte(*b);
         }
         editor.delete_char();
-        assert_eq!(editor.rows[0].chars, b"Hello world");
+        assert_row_chars_equal(&editor, &[b"Hello world"]);
         editor.move_cursor(&AKey::Left, true);
         editor.move_cursor(&AKey::Left, false);
         editor.move_cursor(&AKey::Left, false);
         editor.delete_char();
-        assert_eq!(editor.rows[0].chars, b"Helo world");
+        assert_row_chars_equal(&editor, &[b"Helo world"]);
+    }
+
+    #[test]
+    fn editor_delete_next_char() {
+        let mut editor = Editor::default();
+        for &b in b"Hello world!\nHappy New Year!" {
+            editor.process_keypress(&Key::Char(b));
+        }
+        editor.process_keypress(&Key::Delete);
+        assert_row_chars_equal(&editor, &[b"Hello world!", b"Happy New Year!"]);
+        editor.move_cursor(&AKey::Left, true);
+        editor.process_keypress(&Key::Delete);
+        assert_row_chars_equal(&editor, &[b"Hello world!", b"Happy New ear!"]);
+        editor.move_cursor(&AKey::Left, true);
+        editor.move_cursor(&AKey::Left, true);
+        editor.move_cursor(&AKey::Left, true);
+        editor.process_keypress(&Key::Delete);
+        assert_row_chars_equal(&editor, &[b"Hello world!Happy New ear!"]);
     }
 
     #[test]
     fn editor_move_cursor_left() {
         let mut editor = Editor::default();
-        for b in b"Hello world!\nHappy New Year!" {
-            if *b == b'\n' {
-                editor.insert_new_line();
-            } else {
-                editor.insert_byte(*b);
-            }
+        for &b in b"Hello world!\nHappy New Year!" {
+            editor.process_keypress(&Key::Char(b));
         }
 
         // check current position
@@ -991,12 +1031,8 @@ mod tests {
     #[test]
     fn editor_move_cursor_up() {
         let mut editor = Editor::default();
-        for b in b"abcdefgh\nij\nklmnopqrstuvwxyz" {
-            if *b == b'\n' {
-                editor.insert_new_line();
-            } else {
-                editor.insert_byte(*b);
-            }
+        for &b in b"abcdefgh\nij\nklmnopqrstuvwxyz" {
+            editor.process_keypress(&Key::Char(b));
         }
 
         // check current position
@@ -1019,12 +1055,8 @@ mod tests {
     #[test]
     fn editor_move_cursor_right() {
         let mut editor = Editor::default();
-        for b in b"Hello world\nHappy New Year" {
-            if *b == b'\n' {
-                editor.insert_new_line();
-            } else {
-                editor.insert_byte(*b);
-            }
+        for &b in b"Hello world\nHappy New Year" {
+            editor.process_keypress(&Key::Char(b));
         }
 
         // check current position
@@ -1051,17 +1083,17 @@ mod tests {
         editor.move_cursor(&AKey::Right, true);
         assert_eq!(editor.cursor.x, 11);
         assert_eq!(editor.cursor.y, 0);
+
+        editor.move_cursor(&AKey::Right, false);
+        assert_eq!(editor.cursor.x, 0);
+        assert_eq!(editor.cursor.y, 1);
     }
 
     #[test]
     fn editor_move_cursor_down() {
         let mut editor = Editor::default();
-        for b in b"abcdefgh\nij\nklmnopqrstuvwxyz" {
-            if *b == b'\n' {
-                editor.insert_new_line();
-            } else {
-                editor.insert_byte(*b);
-            }
+        for &b in b"abcdefgh\nij\nklmnopqrstuvwxyz" {
+            editor.process_keypress(&Key::Char(b));
         }
 
         // check current position
@@ -1103,12 +1135,8 @@ mod tests {
     #[test]
     fn editor_press_home_key() {
         let mut editor = Editor::default();
-        for b in b"Hello\nWorld\nand\nFerris!" {
-            if *b == b'\n' {
-                editor.insert_new_line();
-            } else {
-                editor.insert_byte(*b);
-            }
+        for &b in b"Hello\nWorld\nand\nFerris!" {
+            editor.process_keypress(&Key::Char(b));
         }
 
         // check current position
@@ -1138,12 +1166,8 @@ mod tests {
     #[test]
     fn editor_press_end_key() {
         let mut editor = Editor::default();
-        for b in b"Hello\nWorld\nand\nFerris!" {
-            if *b == b'\n' {
-                editor.insert_new_line();
-            } else {
-                editor.insert_byte(*b);
-            }
+        for &b in b"Hello\nWorld\nand\nFerris!" {
+            editor.process_keypress(&Key::Char(b));
         }
 
         // check current position
@@ -1166,6 +1190,159 @@ mod tests {
         assert_eq!(editor.cursor.y, 0);
     }
 
+    #[test]
+    fn editor_page_up_moves_cursor_to_viewport_top() {
+        let mut editor = Editor { screen_rows: 4, ..Default::default() };
+        for _ in 0..10 {
+            editor.insert_new_line();
+        }
+
+        (editor.cursor.y, editor.cursor.x) = (3, 0);
+        editor.insert_byte(b'a');
+        editor.insert_byte(b'b');
+
+        (editor.cursor.y, editor.cursor.x, editor.cursor.roff) = (9, 5, 7);
+        let (should_quit, prompt_mode) = editor.process_keypress(&Key::PageUp);
+
+        assert!(!should_quit);
+        assert!(prompt_mode.is_none());
+        assert_eq!(editor.cursor.y, 3);
+        assert_eq!(editor.cursor.x, 2);
+    }
+
+    #[test]
+    fn editor_page_down_moves_cursor_to_viewport_bottom() {
+        let mut editor = Editor { screen_rows: 4, ..Default::default() };
+        for _ in 0..12 {
+            editor.insert_new_line();
+        }
+
+        (editor.cursor.y, editor.cursor.x) = (11, 0);
+        editor.insert_byte(b'x');
+        editor.insert_byte(b'y');
+        editor.insert_byte(b'z');
+
+        (editor.cursor.x, editor.cursor.roff) = (6, 4);
+        let (should_quit, prompt_mode) = editor.process_keypress(&Key::PageDown);
+        assert!(!should_quit);
+        assert!(prompt_mode.is_none());
+        assert_eq!(editor.cursor.y, 11);
+        assert_eq!(editor.cursor.x, 3);
+
+        (editor.cursor.x, editor.cursor.roff) = (9, 11);
+        let (should_quit, prompt_mode_again) = editor.process_keypress(&Key::PageDown);
+        assert!(!should_quit);
+        assert!(prompt_mode_again.is_none());
+        assert_eq!(editor.cursor.y, editor.rows.len());
+        assert_eq!(editor.cursor.x, 0);
+    }
+
+    #[rstest]
+    #[case::beginning_of_first_row(b"Hello\nWorld!\n", (0, 0), &[&b"World!"[..], &b""[..]], 0)]
+    #[case::middle_of_first_row(b"Hello\nWorld!\n", (3, 0), &[&b"World!"[..], &b""[..]], 0)]
+    #[case::end_of_first_row(b"Hello\nWorld!\n", (5, 0), &[&b"World!"[..], &b""[..]], 0)]
+    #[case::empty_first_row(b"\nHello", (0, 0), &[&b"Hello"[..]], 0)]
+    #[case::beginning_of_only_row(b"Hello", (0, 0), &[&b""[..]], 0)]
+    #[case::middle_of_only_row(b"Hello", (3, 0), &[&b""[..]], 0)]
+    #[case::end_of_only_row(b"Hello", (5, 0), &[&b""[..]], 0)]
+    #[case::beginning_of_middle_row(b"Hello\nWorld!\n", (0, 1), &[&b"Hello"[..], &b""[..]], 1)]
+    #[case::middle_of_middle_row(b"Hello\nWorld!\n", (3, 1), &[&b"Hello"[..], &b""[..]], 1)]
+    #[case::end_of_middle_row(b"Hello\nWorld!\n", (6, 1), &[&b"Hello"[..], &b""[..]], 1)]
+    #[case::empty_middle_row(b"Hello\n\nWorld!", (0, 1), &[&b"Hello"[..], &b"World!"[..]], 1)]
+    #[case::beginning_of_last_row(b"Hello\nWorld!", (0, 1), &[&b"Hello"[..]], 0)]
+    #[case::middle_of_last_row(b"Hello\nWorld!", (3, 1), &[&b"Hello"[..]], 0)]
+    #[case::end_of_last_row(b"Hello\nWorld!", (6, 1), &[&b"Hello"[..]], 0)]
+    #[case::empty_last_row(b"Hello\n", (0, 1), &[&b"Hello"[..]], 0)]
+    #[case::after_last_row(b"Hello\nWorld!", (0, 2), &[&b"Hello"[..], &b"World!"[..]], 2)]
+    fn delete_current_row_updates_buffer_and_position(
+        #[case] initial_buffer: &[u8], #[case] cursor_position: (usize, usize),
+        #[case] expected_rows: &[&[u8]], #[case] expected_cursor_row: usize,
+    ) {
+        let mut editor = Editor::default();
+        for &b in initial_buffer {
+            editor.process_keypress(&Key::Char(b));
+        }
+        (editor.cursor.x, editor.cursor.y) = cursor_position;
+
+        editor.delete_current_row();
+
+        assert_row_chars_equal(&editor, expected_rows);
+        assert_eq!(
+            (editor.cursor.x, editor.cursor.y),
+            (0, expected_cursor_row),
+            "cursor is at {}:{}, expected {}:0",
+            editor.cursor.y,
+            editor.cursor.x,
+            expected_cursor_row
+        );
+    }
+
+    #[rstest]
+    #[case::first_row(0)]
+    #[case::middle_row(5)]
+    #[case::last_row(9)]
+    fn delete_current_row_updates_screen_cols_and_ln_pad(#[case] current_row: usize) {
+        let mut editor = Editor { window_width: 100, ..Default::default() };
+        for _ in 0..10 {
+            editor.insert_new_line();
+        }
+        assert_eq!(editor.screen_cols, 96);
+        assert_eq!(editor.ln_pad, 4);
+
+        editor.cursor.y = current_row;
+        editor.delete_current_row();
+
+        assert_eq!(editor.screen_cols, 97);
+        assert_eq!(editor.ln_pad, 3);
+    }
+
+    #[test]
+    fn delete_current_row_updates_syntax_highlighting() {
+        let mut editor = Editor {
+            syntax: SyntaxConf {
+                ml_comment_delims: Some(("/*".to_owned(), "*/".to_owned())),
+                ..Default::default()
+            },
+            ..Default::default()
+        };
+        for &b in b"A\nb/*c\nd\ne\nf*/g\nh" {
+            editor.process_keypress(&Key::Char(b));
+        }
+
+        assert_row_chars_equal(&editor, &[b"A", b"b/*c", b"d", b"e", b"f*/g", b"h"]);
+        assert_row_synthax_highlighting_types_equal(&editor, &[
+            &[HlType::Normal],
+            &[HlType::Normal, HlType::MlComment, HlType::MlComment, HlType::MlComment],
+            &[HlType::MlComment],
+            &[HlType::MlComment],
+            &[HlType::MlComment, HlType::MlComment, HlType::MlComment, HlType::Normal],
+            &[HlType::Normal],
+        ]);
+
+        (editor.cursor.x, editor.cursor.y) = (0, 4);
+        editor.delete_current_row();
+
+        assert_row_chars_equal(&editor, &[b"A", b"b/*c", b"d", b"e", b"h"]);
+        assert_row_synthax_highlighting_types_equal(&editor, &[
+            &[HlType::Normal],
+            &[HlType::Normal, HlType::MlComment, HlType::MlComment, HlType::MlComment],
+            &[HlType::MlComment],
+            &[HlType::MlComment],
+            &[HlType::MlComment],
+        ]);
+
+        (editor.cursor.x, editor.cursor.y) = (0, 1);
+        editor.delete_current_row();
+
+        assert_row_chars_equal(&editor, &[b"A", b"d", b"e", b"h"]);
+        assert_row_synthax_highlighting_types_equal(&editor, &[
+            &[HlType::Normal],
+            &[HlType::Normal],
+            &[HlType::Normal],
+            &[HlType::Normal],
+        ]);
+    }
+
     #[test]
     fn loop_until_keypress() -> Result<(), Error> {
         let mut editor = Editor::default();
@@ -1238,4 +1415,25 @@ mod tests {
             .and_then(|prompt_mode| prompt_mode.process_keypress(&mut ed, &Key::Char(b'\r')));
         assert_eq!(prompt_mode, None);
     }
+
+    #[rstest]
+    #[case(100, true, 12345, "\u{1b}[38;5;240m12345 │\u{1b}[m")]
+    #[case(100, true, "~", "\u{1b}[38;5;240m~ │\u{1b}[m")]
+    #[case(10, true, 12345, "")]
+    #[case(10, true, "~", "")]
+    #[case(100, false, 12345, "12345 │")]
+    #[case(100, false, "~", "~ │")]
+    #[case(10, false, 12345, "")]
+    #[case(10, false, "~", "")]
+    fn draw_left_padding<T: Display>(
+        #[case] window_width: usize, #[case] use_color: bool, #[case] value: T,
+        #[case] expected: &'static str,
+    ) {
+        let mut editor = Editor { window_width, use_color, ..Default::default() };
+        editor.update_screen_cols();
+
+        let mut buffer = String::new();
+        editor.draw_left_padding(&mut buffer, value);
+        assert_eq!(buffer, expected);
+    }
 }
